﻿#ifndef QS_DEFINE_H
#define QS_DEFINE_H

#if defined(WIN32)
#include <windows.h>
#else
#include <dlfcn.h>
#include <stdio.h>
#endif
#include <stdint.h>
#include <stdlib.h>
#include <pthread.h>
#include <utility>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <vector>
#include <signal.h>
#include <dlfcn.h>
#include <string.h>

#if defined(__clang__)|| defined(__GNUC__)
#define QS_DLLEXPORT __attribute__ ((visibility("default")))
#if __cplusplus == 201703L || _MSVC_LANG == 201703L
//#define QS_SELECTANY __attribute__((selectany))
#define QS_SELECTANY inline
#else
#define QS_SELECTANY 
#endif
#else
typedef __int64 int64_t;
#define QS_DLLEXPORT __declspec(dllexport)
#define QS_SELECTANY __declspec(selectany)
#endif

#if __SIZEOF_LONG__ == 8
#define QS_CALL_TYPE 
#else
#if defined(_WIN32)
#define QS_CALL_TYPE __stdcall
#else
#define QS_CALL_TYPE 
#endif
#endif

typedef struct _qsRect {
    int x;
    int y;
    int w;
    int h;

#if defined(__cplusplus)
    _qsRect()
    {
        x = 0;
        y = 0;
        w = 0;
        h = 0;
    }
    _qsRect(int xVal, int yVal, int wVal, int hVal)
    {
        x = xVal;
        y = yVal;
        w = wVal;
        h = hVal;
    }
    _qsRect(const _qsRect& other)
    {
        x = other.x;
        y = other.y;
        w = other.w;
        h = other.h;
    }
#endif
} qsRect;

typedef struct _qsPoint {
    int x;
    int y;
} qsPoint;

typedef struct _qsSize {
    int w;
    int h;
} qsSize;

typedef enum {
    QS_LBUTTON = 0x01,
    QS_RBUTTON = 0x02,
    QS_SHIFT = 0x04,
    QS_CONTROL = 0x08,
    QS_MBUTTON = 0x10,
} qsMouseFlags;

typedef enum {
    QS_EXTENDED = 0x0100,
    QS_REPEAT = 0x4000,
} qsKeyFlags;

typedef enum {
    QS_MSG_MOUSEMOVE = 0x0200,
    QS_MSG_LBUTTONDOWN = 0x0201,
    QS_MSG_LBUTTONUP = 0x0202,
    QS_MSG_LBUTTONDBLCLK = 0x0203,
    QS_MSG_RBUTTONDOWN = 0x0204,
    QS_MSG_RBUTTONUP = 0x0205,
    QS_MSG_RBUTTONDBLCLK = 0x0206,
    QS_MSG_MBUTTONDOWN = 0x0207,
    QS_MSG_MBUTTONUP = 0x0208,
    QS_MSG_MBUTTONDBLCLK = 0x0209,
    QS_MSG_MOUSEWHEEL = 0x020A,
} qsMouseMsg;

#if !defined(__cplusplus)
#ifndef HAVE_WCHAR_T
typedef unsigned short WCHAR;
#endif
#endif

#include <stdbool.h>

#if defined(__cplusplus)
#define QS_EXTERN_C extern "C" 
#else
#define QS_EXTERN_C 
#endif

typedef char utf8;

typedef enum {
    QS_PROXY_NONE,
    QS_PROXY_HTTP,
    QS_PROXY_SOCKS4,
    QS_PROXY_SOCKS4A,
    QS_PROXY_SOCKS5,
    QS_PROXY_SOCKS5HOSTNAME
} qsProxyType;

typedef struct {
    qsProxyType type;
    char hostname[100];
    unsigned short port;
    char username[50];
    char password[50];
} qsProxy;

typedef enum _qsSettingMask {
    QS_SETTING_PROXY = 1,
    QS_ENABLE_NODEJS = 1 << 3,
    QS_ENABLE_DISABLE_H5VIDEO = 1 << 4,
    QS_ENABLE_DISABLE_PDFVIEW = 1 << 5,
    QS_ENABLE_DISABLE_CC = 1 << 6,
    QS_ENABLE_ENABLE_EGLGLES2 = 1 << 7, // 测试功能，请勿使用
    QS_ENABLE_ENABLE_SWIFTSHAER = 1 << 8, // 测试功能，请勿使用
} qsSettingMask;

typedef void(QS_CALL_TYPE* qsOnBlinkThreadInitCallback)(void* param);

#define QS_Version 20240127
#define QS_MaxVersion 20600319

typedef struct _qsLicense {
    char appID[21];
    char appSecret[36];
    char appVer[16];
} qsLicense;

typedef struct _qsSettings {
    qsLicense license;
    qsProxy proxy;
    unsigned int mask; // 208 offset
    qsOnBlinkThreadInitCallback blinkThreadInitCallback;
    void* blinkThreadInitCallbackParam;
    intptr_t version;
    const char16_t* mainDllPath;
    void* mainDllHandle;
    const char* config;
} qsSettings;

typedef struct _qsViewSettings {
    int size;
    unsigned int bgColor;
} qsViewSettings;

typedef void* qsWebFrameHandle;
typedef void* qsNetJob;

#if defined(__cplusplus)
namespace qs { class QSWebView; }
#else
struct _tagMbWebView;
#endif

#if __SIZEOF_LONG__ == 8
typedef int64_t          qsWebView;
#else
typedef intptr_t         qsWebView;
#endif

#define NULL_WEBVIEW     0

typedef bool(*qsCookieVisitor)(
    void* params,
    const char* name,
    const char* value,
    const char* domain,
    const char* path, // If |path| is non-empty only URLs at or below the path will get the cookie value.
    int secure, // If |secure| is true the cookie will only be sent for HTTPS requests.
    int httpOnly, // If |httponly| is true the cookie will only be sent for HTTP requests.
    int* expires // The cookie expiration date is only valid if |has_expires| is true.
    );

typedef enum {
    QS_CookieCommandClearAllCookies,
    QS_CookieCommandClearSessionCookies,
    QS_CookieCommandFlushCookiesToFile,
    QS_CookieCommandReloadCookiesFromFile,
} qsCookieCommand;

typedef enum {
    QS_NAVIGATION_TYPE_LINKCLICK,
    QS_NAVIGATION_TYPE_FORMSUBMITTE,
    QS_NAVIGATION_TYPE_BACKFORWARD,
    QS_NAVIGATION_TYPE_RELOAD,
    QS_NAVIGATION_TYPE_FORMRESUBMITT,
    QS_NAVIGATION_TYPE_OTHER
} qsNavigationType;

typedef enum {
    QS_CursorInfoPointer,
    QS_CursorInfoCross,
    QS_CursorInfoHand,
    QS_CursorInfoIBeam,
    QS_CursorInfoWait,
    QS_CursorInfoHelp,
    QS_CursorInfoEastResize,
    QS_CursorInfoNorthResize,
    QS_CursorInfoNorthEastResize,
    QS_CursorInfoNorthWestResize,
    QS_CursorInfoSouthResize,
    QS_CursorInfoSouthEastResize,
    QS_CursorInfoSouthWestResize,
    QS_CursorInfoWestResize,
    QS_CursorInfoNorthSouthResize,
    QS_CursorInfoEastWestResize,
    QS_CursorInfoNorthEastSouthWestResize,
    QS_CursorInfoNorthWestSouthEastResize,
    QS_CursorInfoColumnResize,
    QS_CursorInfoRowResize,
    QS_CursorInfoMiddlePanning,
    QS_CursorInfoEastPanning,
    QS_CursorInfoNorthPanning,
    QS_CursorInfoNorthEastPanning,
    QS_CursorInfoNorthWestPanning,
    QS_CursorInfoSouthPanning,
    QS_CursorInfoSouthEastPanning,
    QS_CursorInfoSouthWestPanning,
    QS_CursorInfoWestPanning,
    QS_CursorInfoMove,
    QS_CursorInfoVerticalText,
    QS_CursorInfoCell,
    QS_CursorInfoContextMenu,
    QS_CursorInfoAlias,
    QS_CursorInfoProgress,
    QS_CursorInfoNoDrop,
    QS_CursorInfoCopy,
    QS_CursorInfoNone,
    QS_CursorInfoNotAllowed,
    QS_CursorInfoZoomIn,
    QS_CursorInfoZoomOut,
    QS_CursorInfoGrab,
    QS_CursorInfoGrabbing,
    QS_CursorInfoCustom
} qsCursorInfoType;

typedef struct {
    int x;
    int y;
    int width;
    int height;

    bool menuBarVisible;
    bool statusBarVisible;
    bool toolBarVisible;
    bool locationBarVisible;
    bool scrollbarsVisible;
    bool resizable;
    bool fullscreen;
} qsWindowFeatures;

typedef struct _qsPrintSettings {
    int structSize;
    int dpi;
    int width;
    int height;
    int marginTop;
    int marginBottom;
    int marginLeft;
    int marginRight;
    bool isPrintPageHeadAndFooter;
    bool isPrintBackgroud;
    bool isLandscape;
    bool isPrintToMultiPage;
} qsPrintSettings;

struct qsString;
typedef qsString* qsStringPtr;

typedef struct _qsMemBuf {
    int size;
    void* data;
    size_t length;
} qsMemBuf;

typedef struct {
    struct Item {
        enum qsStorageType {
            // String data with an associated MIME type. Depending on the MIME type, there may be
            // optional metadata attributes as well.
            QS_StorageTypeString,
            // Stores the name of one file being dragged into the renderer.
            QS_StorageTypeFilename,
            // An image being dragged out of the renderer. Contains a buffer holding the image data
            // as well as the suggested name for saving the image to.
            QS_StorageTypeBinaryData,
            // Stores the filesystem URL of one file being dragged into the renderer.
            QS_StorageTypeFileSystemFile,
        } storageType;

        // Only valid when storageType == StorageTypeString.
        qsMemBuf* stringType;
        qsMemBuf* stringData;

        // Only valid when storageType == StorageTypeFilename.
        qsMemBuf* filenameData;
        qsMemBuf* displayNameData;

        // Only valid when storageType == StorageTypeBinaryData.
        qsMemBuf* binaryData;

        // Title associated with a link when stringType == "text/uri-list".
        // Filename when storageType == StorageTypeBinaryData.
        qsMemBuf* title;

        // Only valid when storageType == StorageTypeFileSystemFile.
        qsMemBuf* fileSystemURL;
        long long fileSystemFileSize;

        // Only valid when stringType == "text/html".
        qsMemBuf* baseURL;
    };

    struct Item* m_itemList;
    int m_itemListLength;

    int m_modifierKeyState; // State of Shift/Ctrl/Alt/Meta keys.
    qsMemBuf* m_filesystemId;
} qsWebDragData;

typedef enum {
    QS_WebDragOperationNone = 0,
    QS_WebDragOperationCopy = 1,
    QS_WebDragOperationLink = 2,
    QS_WebDragOperationGeneric = 4,
    QS_WebDragOperationPrivate = 8,
    QS_WebDragOperationMove = 16,
    QS_WebDragOperationDelete = 32,
    QS_WebDragOperationEvery = 0xffffffff
} qsWebDragOperation;

typedef qsWebDragOperation qsWebDragOperationsMask;

typedef enum {
    QS_RESOURCE_TYPE_MAIN_FRAME = 0,       // top level page
    QS_RESOURCE_TYPE_SUB_FRAME = 1,        // frame or iframe
    QS_RESOURCE_TYPE_STYLESHEET = 2,       // a CSS stylesheet
    QS_RESOURCE_TYPE_SCRIPT = 3,           // an external script
    QS_RESOURCE_TYPE_IMAGE = 4,            // an image (jpg/gif/png/etc)
    QS_RESOURCE_TYPE_FONT_RESOURCE = 5,    // a font
    QS_RESOURCE_TYPE_SUB_RESOURCE = 6,     // an "other" subresource.
    QS_RESOURCE_TYPE_OBJECT = 7,           // an object (or eqsed) tag for a plugin, or a resource that a plugin requested.
    QS_RESOURCE_TYPE_MEDIA = 8,            // a media resource.
    QS_RESOURCE_TYPE_WORKER = 9,           // the main resource of a dedicated worker.
    QS_RESOURCE_TYPE_SHARED_WORKER = 10,   // the main resource of a shared worker.
    QS_RESOURCE_TYPE_PREFETCH = 11,        // an explicitly requested prefetch
    QS_RESOURCE_TYPE_FAVICON = 12,         // a favicon
    QS_RESOURCE_TYPE_XHR = 13,             // a XMLHttpRequest
    QS_RESOURCE_TYPE_PING = 14,            // a ping request for <a ping>
    QS_RESOURCE_TYPE_SERVICE_WORKER = 15,  // the main resource of a service worker.
    QS_RESOURCE_TYPE_LAST_TYPE
} qsResourceType;

typedef enum _qsRequestType {
    QS_RequestTypeInvalidation,
    QS_RequestTypeGet,
    QS_RequestTypePost,
    QS_RequestTypePut,
} qsRequestType;

typedef struct _qsSlist {
    char* data;
    struct _qsSlist* next;
} qsSlist;

typedef enum _qsMenuItemId {
    QS_MenuSelectedAllId = 1 << 1,
    QS_MenuSelectedTextId = 1 << 2,
    QS_MenuUndoId = 1 << 3,
    QS_MenuCopyImageId = 1 << 4,
    QS_MenuInspectElementAtId = 1 << 5,
    QS_MenuCutId = 1 << 6,
    QS_MenuPasteId = 1 << 7,
    QS_MenuPrintId = 1 << 8,
    QS_MenuGoForwardId = 1 << 9,
    QS_MenuGoBackId = 1 << 10,
    QS_MenuReloadId = 1 << 11,
} qsMenuItemId;

typedef void* qsWebSocketChannel;

typedef struct _qsWebsocketHookCallbacks {
    qsStringPtr(QS_CALL_TYPE* onWillConnect)(qsWebView webView, void* param, qsWebSocketChannel channel, const utf8* url, bool* needHook);

    bool(QS_CALL_TYPE* onConnected)(qsWebView webView, void* param, qsWebSocketChannel channel);
    qsStringPtr(QS_CALL_TYPE* onReceive)(qsWebView webView, void* param, qsWebSocketChannel channel, int opCode, const char* buf, size_t len, bool* isContinue);
    qsStringPtr(QS_CALL_TYPE* onSend)(qsWebView webView, void* param, qsWebSocketChannel channel, int opCode, const char* buf, size_t len, bool* isContinue);
    void(QS_CALL_TYPE* onError)(qsWebView webView, void* param, qsWebSocketChannel channel);
} qsWebsocketHookCallbacks;


typedef enum {
    QS_JsTypeNumber = 0,
    QS_JsTypeString = 1,
    QS_JsTypeBool = 2,
    //QS_JsTypeObject = 3,
    //QS_JsTypeFunction = 4,
    QS_JsTypeUndefined  = 5,
    //QS_JsTypeArray = 6,
    QS_JsTypeNull = 7,
    QS_JsTypeV8Value = 8,
} qsJsType;

//typedef long long int64_t;
typedef long long qsJsValue;
typedef void* qsJsExecState;

typedef void(QS_CALL_TYPE *qsOnGetPdfPageDataCallback)(qsWebView webView, void* param, void* data, size_t size);

typedef void(QS_CALL_TYPE *qsRunJsCallback)(qsWebView webView, void* param, qsJsExecState es, qsJsValue v);
typedef void(QS_CALL_TYPE* qsJsQueryCallback)(qsWebView webView, void* param, qsJsExecState es, int64_t queryId, int customMsg, const utf8* request);

typedef void(QS_CALL_TYPE *qsTitleChangedCallback)(qsWebView webView, void* param, const utf8* title);
typedef void(QS_CALL_TYPE *qsMouseOverUrlChangedCallback)(qsWebView webView, void* param, const utf8* url);
typedef void(QS_CALL_TYPE *qsURLChangedCallback)(qsWebView webView, void* param, const utf8* url, bool canGoBack, bool canGoForward);
typedef void(QS_CALL_TYPE *qsURLChangedCallback2)(qsWebView webView, void* param, qsWebFrameHandle frameId, const utf8* url);
typedef void(QS_CALL_TYPE *qsPaintUpdatedCallback)(qsWebView webView, void* param, const void* hdc, int x, int y, int cx, int cy);
typedef void(QS_CALL_TYPE* qsAcceleratedPaintCallback)(qsWebView webView, void* param, int type, const qsRect* dirytRects, const size_t dirytRectsSize,void* sharedHandle);
typedef void(QS_CALL_TYPE *qsPaintBitUpdatedCallback)(qsWebView webView, void* param, const void* buffer, const qsRect* r, int width, int height);
typedef void(QS_CALL_TYPE *qsAlertBoxCallback)(qsWebView webView, void* param, const utf8* msg);
typedef bool(QS_CALL_TYPE *qsConfirmBoxCallback)(qsWebView webView, void* param, const utf8* msg);
typedef qsStringPtr(QS_CALL_TYPE *qsPromptBoxCallback)(qsWebView webView, void* param, const utf8* msg, const utf8* defaultResult);
typedef bool(QS_CALL_TYPE *qsNavigationCallback)(qsWebView webView, void* param, qsNavigationType navigationType, const utf8* url);
typedef qsWebView(QS_CALL_TYPE *qsCreateViewCallback)(qsWebView webView, void* param, qsNavigationType navigationType, const utf8* url, const qsWindowFeatures* windowFeatures);
typedef void(QS_CALL_TYPE *qsDocumentReadyCallback)(qsWebView webView, void* param, qsWebFrameHandle frameId);
typedef void(QS_CALL_TYPE *qsLoadUrlFinishCallback)(qsWebView webView, void* param, const utf8* url, qsNetJob job, int len);
typedef void(QS_CALL_TYPE *qsLoadUrlHeadersReceivedCallback)(qsWebView webView, void* param, const char* url, qsNetJob job);
typedef bool(QS_CALL_TYPE *qsCloseCallback)(qsWebView webView, void* param, void* unuse);
typedef bool(QS_CALL_TYPE *qsDestroyCallback)(qsWebView webView, void* param, void* unuse);
typedef void(QS_CALL_TYPE *qsOnShowDevtoolsCallback)(qsWebView webView, void* param);
typedef void(QS_CALL_TYPE *qsDidCreateScriptContextCallback)(qsWebView webView, void* param, qsWebFrameHandle frameId, void* context, int extensionGroup, int worldId);
typedef bool(QS_CALL_TYPE *qsGetPluginListCallback)(bool refresh, void* pluginListBuilder, void* param);
typedef bool(QS_CALL_TYPE *qsNetResponseCallback)(qsWebView webView, void* param, const utf8* url, qsNetJob job);
typedef void(QS_CALL_TYPE* qsThreadCallback)(void* param1, void* param2);
typedef void(QS_CALL_TYPE* qsNodeOnCreateProcessCallback)(qsWebView webView, void* param, const char16_t* applicationPath, const char16_t* arguments, void* startup);

typedef enum {
    QS_LOADING_SUCCEEDED,
    QS_LOADING_FAILED,
    QS_LOADING_CANCELED
} qsLoadingResult;

typedef void(QS_CALL_TYPE *qsLoadingFinishCallback)(qsWebView webView, void* param, qsWebFrameHandle frameId, const utf8* url, qsLoadingResult result, const utf8* failedReason);
typedef bool(QS_CALL_TYPE *qsDownloadCallback)(qsWebView webView, void* param, qsWebFrameHandle frameId, const char* url, void* downloadJob);

typedef enum {
    QS_LevelDebug = 4,
    QS_LevelLog = 1,
    QS_LevelInfo = 5,
    QS_LevelWarning = 2,
    QS_LevelError = 3,
    QS_LevelRevokedError = 6,
    QS_LevelLast = QS_LevelRevokedError
} qsConsoleLevel;
typedef void(QS_CALL_TYPE *qsConsoleCallback)(qsWebView webView, void* param, qsConsoleLevel level, const utf8* message, const utf8* sourceName, unsigned sourceLine, const utf8* stackTrace);

typedef void(QS_CALL_TYPE *qsOnCallUiThread)(qsWebView webView, void* paramOnInThread);
typedef void(QS_CALL_TYPE *qsCallUiThread)(qsWebView webView, qsOnCallUiThread func, void* param);

typedef bool(QS_CALL_TYPE *qsLoadUrlBeginCallback)(qsWebView webView, void* param, const char* url, void* job);
typedef void(QS_CALL_TYPE *qsLoadUrlEndCallback)(qsWebView webView, void* param, const char* url, void* job, void* buf, int len);
typedef void(QS_CALL_TYPE *qsLoadUrlFailCallback)(qsWebView webView, void* param, const char* url, void* job);

typedef void(QS_CALL_TYPE *qsDidCreateScriptContextCallback)(qsWebView webView, void* param, qsWebFrameHandle frameId, void* context, int extensionGroup, int worldId);
typedef void(QS_CALL_TYPE *qsWillReleaseScriptContextCallback)(qsWebView webView, void* param, qsWebFrameHandle frameId, void* context, int worldId);
typedef bool(QS_CALL_TYPE *qsNetResponseCallback)(qsWebView webView, void* param, const char* url, void* job);
typedef void(QS_CALL_TYPE *qsNetGetFaviconCallback)(qsWebView webView, void* param, const utf8* url, qsMemBuf* buf);

enum qsAsynRequestState {
    QS_AsynRequestStateOk = 0,
    QS_AsynRequestStateFail = 1,
};

typedef void(QS_CALL_TYPE* qsCanGoBackForwardCallback)(qsWebView webView, void* param, qsAsynRequestState state, bool b);
typedef void(QS_CALL_TYPE* qsGetCookieCallback)(qsWebView webView, void* param, qsAsynRequestState state, const utf8* cookie);

typedef void* v8ContextPtr;
typedef void* v8Isolate;

typedef void(QS_CALL_TYPE* qsGetSourceCallback)(qsWebView webView, void* param, const utf8* mhtml);
typedef void(QS_CALL_TYPE* qsGetContentAsMarkupCallback)(qsWebView webView, void* param, const utf8* content, size_t size);

typedef struct qsWebUrlRequest* qsWebUrlRequestPtr;
typedef struct qsWebUrlResponse* qsWebUrlResponsePtr;

typedef void(QS_CALL_TYPE* qsOnUrlRequestWillRedirectCallback)(qsWebView webView, void* param, qsWebUrlRequestPtr oldRequest, qsWebUrlRequestPtr request, qsWebUrlResponsePtr redirectResponse);
typedef void(QS_CALL_TYPE* qsOnUrlRequestDidReceiveResponseCallback)(qsWebView webView, void* param, qsWebUrlRequestPtr request, qsWebUrlResponsePtr response);
typedef void(QS_CALL_TYPE* qsOnUrlRequestDidReceiveDataCallback)(qsWebView webView, void* param, qsWebUrlRequestPtr request, const char* data, int dataLength);
typedef void(QS_CALL_TYPE* qsOnUrlRequestDidFailCallback)(qsWebView webView, void* param, qsWebUrlRequestPtr request, const utf8* error);
typedef void(QS_CALL_TYPE* qsOnUrlRequestDidFinishLoadingCallback)(qsWebView webView, void* param, qsWebUrlRequestPtr request, double finishTime);

typedef struct _qsUrlRequestCallbacks {
    qsOnUrlRequestWillRedirectCallback willRedirectCallback;
    qsOnUrlRequestDidReceiveResponseCallback didReceiveResponseCallback;
    qsOnUrlRequestDidReceiveDataCallback didReceiveDataCallback;
    qsOnUrlRequestDidFailCallback didFailCallback;
    qsOnUrlRequestDidFinishLoadingCallback didFinishLoadingCallback;
} qsUrlRequestCallbacks;

typedef enum _qsDownloadOpt {
    QS_DownloadOptCancel,
    QS_DownloadOptCacheData,
} qsDownloadOpt;

typedef void(QS_CALL_TYPE* qsNetJobDataRecvCallback)(void* ptr, qsNetJob job, const char* data, int length);
typedef void(QS_CALL_TYPE* qsNetJobDataFinishCallback)(void* ptr, qsNetJob job, qsLoadingResult result);

typedef struct _qsNetJobDataBind {
    void* param;
    qsNetJobDataRecvCallback recvCallback;
    qsNetJobDataFinishCallback finishCallback;
} qsNetJobDataBind;

typedef void(QS_CALL_TYPE* qsPopupDialogSaveNameCallback)(void* ptr, const char16_t* filePath);

typedef struct _qsDownloadBind {
    void* param;
    qsNetJobDataRecvCallback recvCallback;
    qsNetJobDataFinishCallback finishCallback;
    qsPopupDialogSaveNameCallback saveNameCallback;
} qsDownloadBind;

typedef struct _qsFileFilter {
    const utf8* name; // 例如"image"、"Movies"
    const utf8* extensions; // 例如"jpg|png|gif"
} qsFileFilter;

enum qsDialogProperties {
    QS_DialogPropertiesOpenFile = 1 << 1, // 允许选择文件
    QS_DialogPropertiesOpenDirectory = 1 << 2, // 允许选择文件夹
    QS_DialogPropertiesMultiSelections = 1 << 3, // 允许多选。
    QS_DialogPropertiesShowHiddenFiles = 1 << 4, // 显示对话框中的隐藏文件。
    QS_DialogPropertiesCreateDirectory = 1 << 5, // macOS - 允许你通过对话框的形式创建新的目录。
    QS_DialogPropertiesPromptToCreate = 1 << 6, // Windows - 如果输入的文件路径在对话框中不存在, 则提示创建。 这并不是真的在路径上创建一个文件，而是允许返回一些不存在的地址交由应用程序去创建。
    QS_DialogPropertiesNoResolveAliases = 1 << 7, // macOS - 禁用自动的别名路径(符号链接) 解析。 所选别名现在将会返回别名路径而非其目标路径。
    QS_DialogPropertiesTreatPackageAsDirectory = 1 << 8, // macOS - 将包(如.app 文件夹) 视为目录而不是文件。
    QS_DialogPropertiesDontAddToRecent = 1 << 9, // Windows - 不要将正在打开的项目添加到最近的文档列表中。
};

typedef struct _qsDialogOptions {
    int magic; // 'qsdo'
    const utf8* title;
    const utf8* defaultPath;
    const utf8* buttonLabel;
    qsFileFilter* filters;
    int filtersCount;
    qsDialogProperties prop;
    const utf8* message;
    bool securityScopedBookmarks;
} qsDialogOptions;

typedef struct _qsDownloadOptions {
    int magic; // 'qsdo'
    bool saveAsPathAndName;
} qsDownloadOptions;

typedef qsDownloadOpt(QS_CALL_TYPE* qsDownloadInBlinkThreadCallback)(
    qsWebView webView, 
    void* param,
    size_t expectedContentLength,
    const char* url, 
    const char* mime, 
    const char* disposition, 
    qsNetJob job, 
    qsNetJobDataBind* dataBind
    );

typedef struct _qsPdfDatas {
    int count;
    size_t* sizes;
    const void** datas;
} qsPdfDatas;

typedef void(QS_CALL_TYPE* qsPrintPdfDataCallback)(qsWebView webview, void* param, const qsPdfDatas* datas);

typedef struct _qsScreenshotSettings {
    int structSize;
    int width;
    int height;
} qsScreenshotSettings;

typedef void(QS_CALL_TYPE* qsPrintBitmapCallback)(qsWebView webview, void* param, const char* data, size_t size);
typedef void(QS_CALL_TYPE* qsOnScreenshot)(qsWebView webView, void* param, const char* data, size_t size);

typedef enum _qsHttBodyElementType {
    QS_HttBodyElementTypeData,
    QS_HttBodyElementTypeFile,
} qsHttBodyElementType;

typedef struct _qsPostBodyElement {
    int size;
    qsHttBodyElementType type;
    qsMemBuf* data;
    qsStringPtr filePath;
    int64_t fileStart;
    int64_t fileLength; // -1 means to the end of the file.
} qsPostBodyElement;

typedef struct _qsPostBodyElements {
    int size;
    qsPostBodyElement** element;
    size_t elementSize;
    bool isDirty;
} qsPostBodyElements;

//qswindow-----------------------------------------------------------------------------------
typedef enum _qsWindowType {
    QS_WINDOW_TYPE_POPUP,
    QS_WINDOW_TYPE_TRANSPARENT,
    QS_WINDOW_TYPE_CONTROL
} qsWindowType;

typedef enum _qsWindowInfo {
    QS_WINDOW_INFO_SHARTD_TEXTURE_ENABLE = 1 << 16,
} qsWindowInfo;

typedef bool(QS_CALL_TYPE *qsWindowClosingCallback)(qsWebView webview, void* param);
typedef void(QS_CALL_TYPE *qsWindowDestroyCallback)(qsWebView webview, void* param);

typedef struct _qsDraggableRegion {
    qsRect bounds;
    bool draggable;
} qsDraggableRegion;

typedef void(QS_CALL_TYPE *qsDraggableRegionsChangedCallback)(qsWebView webview, void* param, const qsDraggableRegion* rects, int rectCount);

typedef enum _qsPrintintStep {
    QS_PrintintStepStart,
    QS_PrintintStepPreview,
    QS_PrintintStepPrinting,
} qsPrintintStep;

typedef struct _qsPrintintSettings {
    int dpi;
    int width;
    int height;
    float scale;
} qsPrintintSettings;

typedef struct _qsDefaultPrinterSettings {
    int structSize; // 默认是4 * 10
    bool isLandscape; // 是否为横向打印格式
    bool isPrintHeadFooter; // 
    bool isPrintBackgroud; // 是否打印背景
    int edgeDistanceLeft; // 上边距单位：毫米
    int edgeDistanceTop;
    int edgeDistanceRight;
    int edgeDistanceBottom;
    int copies; // 默认打印份数
    int paperType; // DMPAPER_A4等
#if defined(__cplusplus)
    inline _qsDefaultPrinterSettings();
#endif
} qsDefaultPrinterSettings;

typedef bool(QS_CALL_TYPE *qsPrintingCallback)(qsWebView webview, void* param, qsPrintintStep step, void* hDC, const qsPrintintSettings* settings, int pageCount);

typedef qsStringPtr(QS_CALL_TYPE *qsImageBufferToDataURLCallback)(qsWebView webView, void* param, const char* data, size_t size);

//JavaScript Bind-----------------------------------------------------------------------------------


#define QS_DEFINE_ITERATOR0(returnVal, name, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR1(returnVal, name, p1, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR2(returnVal, name, p1, p2, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR3(returnVal, name, p1, p2, p3, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR4(returnVal, name, p1, p2, p3, p4, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR5(returnVal, name, p1, p2, p3, p4, p5, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR6(returnVal, name, p1, p2, p3, p4, p5, p6, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR7(returnVal, name, p1, p2, p3, p4, p5, p6, p7, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR8(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7, p8); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR9(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7, p8, p9); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR10(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

#define QS_DEFINE_ITERATOR11(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11); \
    QS_SELECTANY FN_##name name = ((FN_##name)0);

//---

#define QS_DECLARE_H_ITERATOR0(returnVal, name, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR1(returnVal, name, p1, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR2(returnVal, name, p1, p2, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR3(returnVal, name, p1, p2, p3, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR4(returnVal, name, p1, p2, p3, p4, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR5(returnVal, name, p1, p2, p3, p4, p5, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR6(returnVal, name, p1, p2, p3, p4, p5, p6, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR7(returnVal, name, p1, p2, p3, p4, p5, p6, p7, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR8(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7, p8); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR9(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7, p8, p9); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR10(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10); \
    extern FN_##name name;

#define QS_DECLARE_H_ITERATOR11(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, description) \
    typedef returnVal(QS_CALL_TYPE* FN_##name)(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11); \
    extern FN_##name name;

// ---

#define QS_DECLARE_ITERATOR0(returnVal, name, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name();

#define QS_DECLARE_ITERATOR1(returnVal, name, p1, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1);

#define QS_DECLARE_ITERATOR2(returnVal, name, p1, p2, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2);

#define QS_DECLARE_ITERATOR3(returnVal, name, p1, p2, p3, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3);

#define QS_DECLARE_ITERATOR4(returnVal, name, p1, p2, p3, p4, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3, p4);

#define QS_DECLARE_ITERATOR5(returnVal, name, p1, p2, p3, p4, p5, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3, p4, p5);

#define QS_DECLARE_ITERATOR6(returnVal, name, p1, p2, p3, p4, p5, p6, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3, p4, p5, p6);

#define QS_DECLARE_ITERATOR7(returnVal, name, p1, p2, p3, p4, p5, p6, p7, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3, p4, p5, p6, p7);

#define QS_DECLARE_ITERATOR8(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3, p4, p5, p6, p7, p8);

#define QS_DECLARE_ITERATOR9(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3, p4, p5, p6, p7, p8, p9);

#define QS_DECLARE_ITERATOR10(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10);

#define QS_DECLARE_ITERATOR11(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, description) \
    QS_EXTERN_C QS_DLLEXPORT returnVal QS_CALL_TYPE name(p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11);

// ---

#if defined(WIN32)
#define QS_GET_PTR_ITERATOR(name) \
    name = (FN_##name)GetProcAddress(g_hMiniblinkMod, #name); \
    if (!name) \
        MessageBoxA(((void*)0), "qs api not found", #name, 0);
#else
#define QS_GET_PTR_ITERATOR(name) \
    name = (FN_##name)dlsym(g_hMiniblinkMod, #name); \
    if (!name) \
        printf("qs api not found: %s", #name);
#endif

#define QS_GET_PTR_ITERATOR0(returnVal, name, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR1(returnVal, name, p1, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR2(returnVal, name, p1, p2, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR3(returnVal, name, p1, p2, p3, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR4(returnVal, name, p1, p2, p3, p4, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR5(returnVal, name, p1, p2, p3, p4, p5, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR6(returnVal, name, p1, p2, p3, p4, p5, p6, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR7(returnVal, name, p1, p2, p3, p4, p5, p6, p7, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR8(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR9(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR10(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, description) \
    QS_GET_PTR_ITERATOR(name);

#define QS_GET_PTR_ITERATOR11(returnVal, name, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, description) \
    QS_GET_PTR_ITERATOR(name);

// 以下是qs的导出函数。格式按照【返回类型】【函数名】【参数】来排列
#define QS_FOR_EACH_DEFINE_FUNCTION(ITERATOR0, ITERATOR1, ITERATOR2, ITERATOR3, ITERATOR4, ITERATOR5, ITERATOR6, ITERATOR7, ITERATOR8, ITERATOR9, ITERATOR10, ITERATOR11) \
ITERATOR0(void, qsUninit, "") \
ITERATOR0(qsSettings*, qsCreateInitSettings, "方便c#等其他语言创建setting结构体") \
ITERATOR3(void, qsSetInitSettings, qsSettings* settings, const char* name, const char* value, "") \
ITERATOR0(qsWebView, qsCreateWebView, "") \
ITERATOR6(qsWebView, qsCreateWebViewBindGTKWindow, void* rootWindow, void* drawingArea, uint32_t style, uint32_t styleEx, int width, int height, "用于GTK绑定窗口") \
ITERATOR1(void, qsDestroyWebView, qsWebView, "") \
ITERATOR6(qsWebView, qsCreateWebWindow, qsWindowType type, void* parent, int x, int y, int width, int height, "") \
ITERATOR7(qsWebView, qsCreateWebCustomWindow, void* parent, uint32_t style, uint32_t styleEx, int x, int y, int width, int height, "") \
ITERATOR5(void, qsMoveWindow, qsWebView webview, int x, int y, int w, int h, "") \
ITERATOR1(void, qsMoveToCenter, qsWebView webview, "") \
ITERATOR2(void, qsSetAutoDrawToHwnd, qsWebView webview, bool b, "离屏模式下控制是否自动上屏") \
ITERATOR2(void, qsGetCaretRect, qsWebView webviewHandle, qsRect* r, "") \
\
ITERATOR2(void, qsSetAudioMuted, qsWebView webview, bool b, "") \
ITERATOR1(bool, qsIsAudioMuted, qsWebView webview, "") \
\
ITERATOR2(qsStringPtr, qsCreateString, const utf8* str, size_t length, "") \
ITERATOR2(qsStringPtr, qsCreateStringWithoutNullTermination, const utf8* str, size_t length, "") \
ITERATOR1(void, qsDeleteString, qsStringPtr str, "") \
ITERATOR1(size_t, qsGetStringLen, qsStringPtr str, "") \
ITERATOR1(const utf8*, qsGetString, qsStringPtr str, "") \
\
ITERATOR2(void, qsSetProxy, qsWebView webView, const qsProxy* proxy, "") \
ITERATOR3(void, qsSetDebugConfig, qsWebView webView, const char* debugString, const char* param, "") \
\
ITERATOR3(void, qsNetSetData, qsNetJob jobPtr, void* buf, int len, "调用此函数后,网络层收到数据会存储在一buf内,接收数据完成后响应OnLoadUrlEnd事件.#此调用严重影响性能,慎用" \
    "此函数和qsNetSetData的区别是，qsNetHookRequest会在接受到真正网络数据后再调用回调，并允许回调修改网络数据。"\
    "而qsNetSetData是在网络数据还没发送的时候修改") \
ITERATOR1(void, qsNetHookRequest, qsNetJob jobPtr, "") \
ITERATOR2(void, qsNetChangeRequestUrl, qsNetJob jobPtr, const char* url, "") \
ITERATOR1(void, qsNetContinueJob, qsNetJob jobPtr, "") \
ITERATOR1(const qsSlist*, qsNetGetRawHttpHeadInBlinkThread, qsNetJob jobPtr, "") \
ITERATOR1(const qsSlist*, qsNetGetRawResponseHeadInBlinkThread, qsNetJob jobPtr, "") \
ITERATOR1(void, qsNetHoldJobToAsynCommit, qsNetJob jobPtr, "") \
ITERATOR1(void, qsNetCancelRequest, qsNetJob jobPtr, "")\
ITERATOR3(void, qsNetOnResponse, qsWebView webviewHandle, qsNetResponseCallback callback, void* param, "注意此接口的回调是在另外个线程")\
\
ITERATOR3(void, qsNetSetWebsocketCallback, qsWebView webview, const qsWebsocketHookCallbacks* callbacks, void* param, "")\
ITERATOR3(void, qsNetSendWsText, qsWebSocketChannel channel, const char* buf, size_t len, "")\
ITERATOR3(void, qsNetSendWsBlob, qsWebSocketChannel channel, const char* buf, size_t len, "")\
\
ITERATOR2(void, qsNetEnableResPacket, qsWebView webviewHandle, const char16_t* pathName, "")\
\
ITERATOR1(qsPostBodyElements*, qsNetGetPostBody, qsNetJob jobPtr, "") \
ITERATOR2(qsPostBodyElements*, qsNetCreatePostBodyElements, qsWebView webView, size_t length, "") \
ITERATOR1(void, qsNetFreePostBodyElements, qsPostBodyElements* elements, "") \
ITERATOR1(qsPostBodyElement*, qsNetCreatePostBodyElement, qsWebView webView, "") \
ITERATOR1(void, qsNetFreePostBodyElement, qsPostBodyElement* element, "") \
\
ITERATOR3(qsWebUrlRequestPtr, qsNetCreateWebUrlRequest, const utf8* url, const utf8* method, const utf8* mime, "")\
ITERATOR3(void, qsNetAddHTTPHeaderFieldToUrlRequest, qsWebUrlRequestPtr request, const utf8* name, const utf8* value, "")\
ITERATOR4(int, qsNetStartUrlRequest, qsWebView webView, qsWebUrlRequestPtr request, void* param, const qsUrlRequestCallbacks* callbacks, "")\
ITERATOR1(int, qsNetGetHttpStatusCode, qsWebUrlResponsePtr response, "")\
ITERATOR1(qsRequestType, qsNetGetRequestMethod, qsNetJob jobPtr, "")\
ITERATOR1(int64_t, qsNetGetExpectedContentLength, qsWebUrlResponsePtr response, "")\
ITERATOR1(const utf8*, qsNetGetResponseUrl, qsWebUrlResponsePtr response, "")\
ITERATOR1(void, qsNetCancelWebUrlRequest, int requestId, "")\
ITERATOR2(void, qsSetViewProxy, qsWebView webView, const qsProxy* proxy, "")\
ITERATOR2(void, qsNetSetMIMEType, qsNetJob jobPtr, const char* type, "") \
ITERATOR1(const char*, qsNetGetMIMEType, qsNetJob jobPtr, "只能在blink线程调用（非主线程）") \
ITERATOR3(const utf8*, qsNetGetHTTPHeaderField, qsNetJob job, const char* key, bool fromRequestOrResponse, "") \
ITERATOR4(void, qsNetSetHTTPHeaderField, qsNetJob jobPtr, const char16_t* key, const char16_t* value, bool response, "") \
\
ITERATOR2(void, qsSetMouseEnabled, qsWebView webView, bool b, "") \
ITERATOR2(void, qsSetTouchEnabled, qsWebView webView, bool b, "") \
ITERATOR2(void, qsSetSystemTouchEnabled, qsWebView webView, bool b, "") \
ITERATOR2(void, qsSetContextMenuEnabled, qsWebView webView, bool b, "") \
ITERATOR2(void, qsSetNavigationToNewWindowEnable, qsWebView webView, bool b, "") \
ITERATOR2(void, qsSetHeadlessEnabled, qsWebView webView, bool b, "可以关闭渲染") \
ITERATOR2(void, qsSetDragDropEnable, qsWebView webView, bool b, "可以关闭拖拽文件、文字") \
ITERATOR2(void, qsSetDragEnable, qsWebView webView, bool b, "可关闭自动响应WM_DROPFILES消息让网页加载本地文件") \
ITERATOR3(void, qsSetContextMenuItemShow, qsWebView webView, qsMenuItemId item, bool isShow, "设置某项menu是否显示") \
\
ITERATOR2(void, qsSetHandle, qsWebView webView, void* wnd, "") \
ITERATOR3(void, qsSetHandleOffset, qsWebView webView, int x, int y, "") \
ITERATOR1(void*, qsGetHostHWND, qsWebView webView, "") \
ITERATOR2(void, qsSetTransparent, qsWebView webviewHandle, bool transparent, "") \
ITERATOR2(void, qsSetViewSettings, qsWebView webviewHandle, const qsViewSettings* settings, "") \
\
ITERATOR2(void, qsSetCspCheckEnable, qsWebView webView, bool b, "") \
ITERATOR2(void, qsSetNpapiPluginsEnabled, qsWebView webView, bool b, "") \
ITERATOR2(void, qsSetMemoryCacheEnable, qsWebView webView, bool b, "") \
ITERATOR3(void, qsSetCookie, qsWebView webView, const utf8* url, const utf8* cookie, "cookie格式必须是:PRODUCTINFO=webxpress; domain=.fidelity.com; path=/; secure") \
ITERATOR2(void, qsSetCookieEnabled, qsWebView webView, bool enable, "") \
ITERATOR2(void, qsSetCookieJarPath, qsWebView webView, const char16_t* path, "") \
ITERATOR2(void, qsSetCookieJarFullPath, qsWebView webView, const char16_t* path, "") \
ITERATOR2(void, qsSetLocalStorageFullPath, qsWebView webView, const char16_t* path, "") \
ITERATOR1(const utf8*, qsGetTitle, qsWebView webView, "") \
ITERATOR2(void, qsSetWindowTitle, qsWebView webView, const utf8* title, "") \
ITERATOR2(void, qsSetWindowTitleW, qsWebView webView, const char16_t* title, "") \
ITERATOR1(const utf8*, qsGetUrl, qsWebView webView, "") \
ITERATOR1(int, qsGetCursorInfoType, qsWebView webView, "") \
ITERATOR2(void, qsAddPluginDirectory, qsWebView webView, const char16_t* path, "") \
ITERATOR2(void, qsSetUserAgent, qsWebView webView, const utf8* userAgent, "") \
ITERATOR2(void, qsSetZoomFactor, qsWebView webView, float factor, "") \
ITERATOR1(float, qsGetZoomFactor, qsWebView webView, "") \
ITERATOR2(void, qsSetDiskCacheEnabled, qsWebView webView, bool enable, "") \
ITERATOR2(void, qsSetDiskCachePath, qsWebView webView, const char16_t* path, "") \
ITERATOR2(void, qsSetDiskCacheLimit, qsWebView webView, size_t limit, "") \
ITERATOR2(void, qsSetDiskCacheLimitDisk, qsWebView webView, size_t limit, "") \
ITERATOR2(void, qsSetDiskCacheLevel, qsWebView webView, int Level, "") \
\
ITERATOR2(void, qsSetResourceGc, qsWebView webView, int intervalSec, "") \
\
ITERATOR3(void, qsCanGoForward, qsWebView webView, qsCanGoBackForwardCallback callback, void* param, "") \
ITERATOR3(void, qsCanGoBack, qsWebView webView, qsCanGoBackForwardCallback callback, void* param, "") \
ITERATOR3(void, qsGetCookie, qsWebView webView, qsGetCookieCallback callback, void* param, "") \
ITERATOR1(const utf8*, qsGetCookieOnBlinkThread, qsWebView webView, "") \
ITERATOR1(void, qsClearCookie, qsWebView webView, "") \
\
ITERATOR3(void, qsResize, qsWebView webView, int w, int h, "") \
\
ITERATOR3(void, qsOnNavigation, qsWebView webView, qsNavigationCallback callback, void* param, "") \
ITERATOR3(void, qsOnNavigationSync, qsWebView webView, qsNavigationCallback callback, void* param, "") \
ITERATOR3(void, qsOnCreateView, qsWebView webView, qsCreateViewCallback callback, void* param, "") \
ITERATOR3(void, qsOnDocumentReady, qsWebView webView, qsDocumentReadyCallback callback, void* param, "") \
ITERATOR3(void, qsOnPaintUpdated, qsWebView webView, qsPaintUpdatedCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnPaintBitUpdated, qsWebView webView, qsPaintBitUpdatedCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnAcceleratedPaint, qsWebView webView, qsAcceleratedPaintCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnLoadUrlBegin, qsWebView webView, qsLoadUrlBeginCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnLoadUrlEnd, qsWebView webView, qsLoadUrlEndCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnLoadUrlFail, qsWebView webView, qsLoadUrlFailCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnTitleChanged, qsWebView webView, qsTitleChangedCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnURLChanged, qsWebView webView, qsURLChangedCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnLoadingFinish, qsWebView webView, qsLoadingFinishCallback callback, void* param, "") \
ITERATOR3(void, qsOnDownload, qsWebView webView, qsDownloadCallback callback, void* param, "") \
ITERATOR3(void, qsOnDownloadInBlinkThread, qsWebView webView, qsDownloadInBlinkThreadCallback callback, void* param, "") \
ITERATOR3(void, qsOnAlertBox, qsWebView webView, qsAlertBoxCallback callback, void* param, "") \
ITERATOR3(void, qsOnConfirmBox, qsWebView webView, qsConfirmBoxCallback callback, void* param, "") \
ITERATOR3(void, qsOnPromptBox, qsWebView webView, qsPromptBoxCallback callback, void* param, "") \
ITERATOR3(void, qsOnNetGetFavicon, qsWebView webView, qsNetGetFaviconCallback callback, void* param, "") \
ITERATOR3(void, qsOnConsole, qsWebView webView, qsConsoleCallback callback, void* param, "") \
ITERATOR3(bool, qsOnClose, qsWebView webView, qsCloseCallback callback, void* param, "") \
ITERATOR3(bool, qsOnDestroy, qsWebView webView, qsDestroyCallback callback, void* param, "") \
ITERATOR3(bool, qsOnPrinting, qsWebView webView, qsPrintingCallback callback, void* param, "") \
ITERATOR3(void, qsOnDidCreateScriptContext, qsWebView webView, qsDidCreateScriptContextCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnPluginList, qsWebView webView, qsGetPluginListCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnImageBufferToDataURL, qsWebView webView, qsImageBufferToDataURLCallback callback, void* callbackParam, "") \
\
ITERATOR1(void, qsGoBack, qsWebView webView, "") \
ITERATOR1(void, qsGoForward, qsWebView webView, "") \
ITERATOR2(void, qsNavigateAtIndex, qsWebView webView, int index, "") \
ITERATOR1(int, qsGetNavigateIndex, qsWebView webView, "") \
ITERATOR1(void, qsStopLoading, qsWebView webView, "") \
ITERATOR1(void, qsReload, qsWebView webView, "") \
ITERATOR2(void, qsPerformCookieCommand, qsWebView webView, qsCookieCommand command, "") \
\
ITERATOR1(void, qsEditorSelectAll, qsWebView webView, "") \
ITERATOR1(void, qsEditorCopy, qsWebView webView, "") \
ITERATOR1(void, qsEditorCut, qsWebView webView, "") \
ITERATOR1(void, qsEditorPaste, qsWebView webView, "") \
ITERATOR1(void, qsEditorDelete, qsWebView webView, "") \
ITERATOR1(void, qsEditorUndo, qsWebView webView, "") \
\
ITERATOR5(bool, qsFireMouseEvent, qsWebView webView, unsigned int message, int x, int y, unsigned int flags, "") \
ITERATOR4(bool, qsFireContextMenuEvent, qsWebView webView, int x, int y, unsigned int flags, "") \
ITERATOR5(bool, qsFireMouseWheelEvent, qsWebView webView, int x, int y, int delta, unsigned int flags, "") \
ITERATOR4(bool, qsFireKeyUpEvent, qsWebView webView, unsigned int virtualKeyCode, unsigned int flags, bool systemKey, "") \
ITERATOR4(bool, qsFireKeyDownEvent, qsWebView webView, unsigned int virtualKeyCode, unsigned int flags, bool systemKey, "") \
ITERATOR4(bool, qsFireKeyPressEvent, qsWebView webView, unsigned int charCode, unsigned int flags, bool systemKey, "") \
ITERATOR6(bool, qsFireWindowsMessage, qsWebView webView, void* hWnd, unsigned int message, unsigned long long int wParam, long long int lParam, long long int* result, "") \
\
ITERATOR1(void, qsSetFocus, qsWebView webView, "") \
ITERATOR1(void, qsKillFocus, qsWebView webView, "") \
\
ITERATOR2(void, qsShowWindow, qsWebView webview, bool show, "") \
\
ITERATOR2(void, qsLoadURL, qsWebView webView, const utf8* url, "") \
ITERATOR3(void, qsLoadHtmlWithBaseUrl, qsWebView webView, const utf8* html, const utf8* baseUrl, "") \
ITERATOR4(void, qsPostURL, qsWebView webView, const utf8* url, const char* postData, int postLen, "") \
\
ITERATOR1(void*, qsGetLockedViewDC, qsWebView webView, "") \
ITERATOR1(void, qsUnlockViewDC, qsWebView webView, "") \
\
ITERATOR1(void, qsWake, qsWebView webView, "") \
\
ITERATOR2(double, qsJsToDouble, qsJsExecState es, qsJsValue v, "") \
ITERATOR2(bool, qsJsToBoolean, qsJsExecState es, qsJsValue v, "") \
ITERATOR2(const utf8*, qsJsToString, qsJsExecState es, qsJsValue v, "") \
ITERATOR2(qsJsType, qsGetJsValueType, qsJsExecState es, qsJsValue v, "") \
ITERATOR3(void, qsOnJsQuery, qsWebView webView, qsJsQueryCallback callback, void* param, "") \
ITERATOR4(void, qsResponseQuery, qsWebView webView, int64_t queryId, int customMsg, const utf8* response, "") \
ITERATOR7(void, qsRunJs, qsWebView webView, qsWebFrameHandle frameId, const utf8* script, bool isInClosure, qsRunJsCallback callback, void* param, void* unuse, "") \
ITERATOR4(qsJsValue, qsRunJsSync, qsWebView webView, qsWebFrameHandle frameId, const utf8* script, bool isInClosure, "") \
ITERATOR1(qsWebFrameHandle, qsWebFrameGetMainFrame, qsWebView webView, "") \
ITERATOR2(bool, qsIsMainFrame, qsWebView webView, qsWebFrameHandle frameId, "") \
ITERATOR2(void, qsSetNodeJsEnable, qsWebView webView, bool b, "") \
\
ITERATOR5(void, qsSetDeviceParameter, qsWebView webView, const char* device, const char* paramStr, int paramInt, float paramFloat, "") \
\
ITERATOR4(void, qsGetContentAsMarkup, qsWebView webView, qsGetContentAsMarkupCallback calback, void* param, qsWebFrameHandle frameId, "") \
ITERATOR3(void, qsGetSource, qsWebView webView, qsGetSourceCallback calback, void* param, "") \
ITERATOR3(void, qsUtilSerializeToMHTML, qsWebView webView, qsGetSourceCallback calback, void* param, "") \
ITERATOR1(const char*, qsUtilCreateRequestCode, const char* registerInfo, "") \
ITERATOR1(bool, qsUtilIsRegistered, const char16_t* defaultPath, "") \
ITERATOR3(bool, qsUtilPrint, qsWebView webView, qsWebFrameHandle frameId, const qsPrintSettings* printParams, "") \
ITERATOR1(const utf8*, qsUtilBase64Encode, const utf8* str, "") \
ITERATOR1(const utf8*, qsUtilBase64Decode, const utf8* str, "") \
ITERATOR1(const utf8*, qsUtilDecodeURLEscape, const utf8* url, "") \
ITERATOR1(const utf8*, qsUtilEncodeURLEscape, const utf8* url, "") \
ITERATOR1(const qsMemBuf*, qsUtilCreateV8Snapshot, const utf8* str, "") \
ITERATOR5(void, qsUtilPrintToPdf, qsWebView webView, qsWebFrameHandle frameId, const qsPrintSettings* settings, qsPrintPdfDataCallback callback, void* param, "") \
ITERATOR5(void, qsUtilPrintToBitmap, qsWebView webView, qsWebFrameHandle frameId, const qsScreenshotSettings* settings, qsPrintBitmapCallback callback, void* param, "") \
ITERATOR4(void, qsUtilScreenshot, qsWebView webView, const qsScreenshotSettings* settings, qsOnScreenshot callback, void* param, "") \
ITERATOR2(bool, qsUtilsSilentPrint, qsWebView webView, const char* settings, "") \
\
ITERATOR3(bool, qsPopupDownloadMgr, qsWebView webView, const char* url, void* downloadJob, "") \
ITERATOR9(qsDownloadOpt, qsPopupDialogAndDownload, qsWebView webView, const qsDialogOptions* dialogOpt, size_t contentLength, const char* url, \
    const char* mime, const char* disposition, qsNetJob job, qsNetJobDataBind* dataBind, qsDownloadBind* callbackBind, "") \
ITERATOR10(qsDownloadOpt, qsDownloadByPath, qsWebView webView, const qsDownloadOptions* downloadOptions, const char16_t* path, \
    size_t expectedContentLength, const char* url, const char* mime, const char* disposition,qsNetJob job, qsNetJobDataBind* dataBind, qsDownloadBind* callbackBind, "") \
\
ITERATOR3(void, qsGetPdfPageData, qsWebView webView, qsOnGetPdfPageDataCallback callback, void* param, "") \
\
ITERATOR3(qsMemBuf*, qsCreateMemBuf, qsWebView webView, void* buf, size_t length, "") \
ITERATOR1(void, qsFreeMemBuf, qsMemBuf* buf, "") \
\
ITERATOR4(void, qsPluginListBuilderAddPlugin, void* builder, const utf8* name, const utf8* description, const utf8* fileName, "") \
ITERATOR3(void, qsPluginListBuilderAddMediaTypeToLastPlugin, void* builder, const utf8* name, const utf8* description, "") \
ITERATOR2(void, qsPluginListBuilderAddFileExtensionToLastMediaType, void* builder, const utf8* fileExtension, "") \
\
ITERATOR0(void, qsEnableHighDPISupport, "") \
ITERATOR0(void, qsRunMessageLoop, "") \
ITERATOR0(void, qsExitMessageLoop, "") \
ITERATOR3(void, qsOnLoadUrlFinish, qsWebView webView, qsLoadUrlFinishCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnLoadUrlHeadersReceived, qsWebView webView, qsLoadUrlHeadersReceivedCallback callback, void* callbackParam, "") \
ITERATOR3(void, qsOnDocumentReadyInBlinkThread, qsWebView webView, qsDocumentReadyCallback callback, void* param, "") \
ITERATOR2(void, qsUtilSetDefaultPrinterSettings, qsWebView webView, const qsDefaultPrinterSettings* setting, "") \
ITERATOR1(int, qsGetContentWidth, qsWebView webView, "") \
ITERATOR1(int, qsGetContentHeight, qsWebView webView, "") \
ITERATOR0(qsWebView, qsGetWebViewForCurrentContext, "") \
ITERATOR5(bool, qsRegisterEmbedderCustomElement, qsWebView webviewHandle, qsWebFrameHandle frameId, const char* name, void* options, void* outResult, "") \
ITERATOR3(void, qsOnNodeCreateProcess, qsWebView webviewHandle, qsNodeOnCreateProcessCallback callback, void* param, "") \
ITERATOR2(qsJsExecState, qsGetGlobalExecByFrame, qsWebView webView, qsWebFrameHandle frameId, "") \
ITERATOR2(void*, qsJsToV8Value, qsJsExecState es, qsJsValue v, "") \
ITERATOR3(void, qsOnThreadIdle, qsThreadCallback callback, void* param1, void* param2, "") \
ITERATOR3(void, qsOnBlinkThreadInit, qsThreadCallback callback, void* param1, void* param2, "") \
ITERATOR3(void, qsCallBlinkThreadAsync, qsThreadCallback callback, void* param1, void* param2, "") \
ITERATOR3(void, qsCallBlinkThreadSync, qsThreadCallback callback, void* param1, void* param2, "") \
ITERATOR3(void, qsCallUiThreadSync, qsThreadCallback callback, void* param1, void* param2, "") \
ITERATOR3(void, qsCallUiThreadAsync, qsThreadCallback callback, void* param1, void* param2, "") \
ITERATOR3(void, qsSetUserKeyValue, qsWebView webView, const char* key, void* value, "") \
ITERATOR2(void*, qsGetUserKeyValue, qsWebView webView, const char* key, "") \
ITERATOR2(void, qsGoToOffset, qsWebView webView, int offset, "") \
ITERATOR2(void, qsGoToIndex, qsWebView webView, int index, "") \
ITERATOR1(void, qsEditorRedo, qsWebView webView, "") \
ITERATOR1(void, qsEditorUnSelect, qsWebView webView, "") \
ITERATOR0(v8Isolate, qsGetBlinkMainThreadIsolate, "") \
ITERATOR3(void, qsInsertCSSByFrame, qsWebView webView, qsWebFrameHandle frameId, const utf8* cssText, "") \
ITERATOR3(void, qsWebFrameGetMainWorldScriptContext, qsWebView webView, qsWebFrameHandle frameId, v8ContextPtr contextOut, "") \
ITERATOR3(void, qsOnWillReleaseScriptContext, qsWebView webView, qsWillReleaseScriptContextCallback callback, void* callbackParam, "") \
ITERATOR1(const char*, qsNetGetReferrer, qsNetJob jobPtr, "获取request的referrer") \
ITERATOR2(void, qsSetEditable, qsWebView webView, bool editable, "") \
ITERATOR1(void*, qsGetProcAddr, const char* name, "")

#if ENABLE_QS == 1
// 如果是在dll\so里

QS_EXTERN_C QS_DLLEXPORT int QS_CALL_TYPE qsOnlineArgument(const char* key, char value[]);

QS_EXTERN_C QS_DLLEXPORT bool QS_CALL_TYPE qsOnlineTracking(const char* action,
                                                            const char* arg1,
                                                            const char* arg2,
                                                            const char* arg3,
                                                            const char* arg4,
                                                            const char* arg5);

QS_EXTERN_C QS_DLLEXPORT bool QS_CALL_TYPE qsInit(const qsSettings* settings);

QS_FOR_EACH_DEFINE_FUNCTION(QS_DECLARE_ITERATOR0, QS_DECLARE_ITERATOR1, QS_DECLARE_ITERATOR2, \
    QS_DECLARE_ITERATOR3, QS_DECLARE_ITERATOR4, QS_DECLARE_ITERATOR5, QS_DECLARE_ITERATOR6, QS_DECLARE_ITERATOR7, QS_DECLARE_ITERATOR8, QS_DECLARE_ITERATOR9, QS_DECLARE_ITERATOR10, QS_DECLARE_ITERATOR11)

#else // ENABLE_QS

#if QS_IN_EXE_H == 1 || !defined(__clang__)
// 如果在linux exp文件或者win exe里
QS_FOR_EACH_DEFINE_FUNCTION(QS_DEFINE_ITERATOR0, QS_DEFINE_ITERATOR1, QS_DEFINE_ITERATOR2, \
    QS_DEFINE_ITERATOR3, QS_DEFINE_ITERATOR4, QS_DEFINE_ITERATOR5, QS_DEFINE_ITERATOR6, QS_DEFINE_ITERATOR7, QS_DEFINE_ITERATOR8, QS_DEFINE_ITERATOR9, QS_DEFINE_ITERATOR10, QS_DEFINE_ITERATOR11)
#else
// 如果在linux 非exp文件
QS_FOR_EACH_DEFINE_FUNCTION(QS_DECLARE_H_ITERATOR0, QS_DECLARE_H_ITERATOR1, QS_DECLARE_H_ITERATOR2, \
    QS_DECLARE_H_ITERATOR3, QS_DECLARE_H_ITERATOR4, QS_DECLARE_H_ITERATOR5, QS_DECLARE_H_ITERATOR6, QS_DECLARE_H_ITERATOR7, QS_DECLARE_H_ITERATOR8, \
    QS_DECLARE_H_ITERATOR9, QS_DECLARE_H_ITERATOR10, QS_DECLARE_H_ITERATOR11)
#endif // QS_IN_EXE_H

typedef int (QS_CALL_TYPE *FN_qsOnlineArgument)(const char* key, char value[]);

typedef bool (QS_CALL_TYPE *FN_qsOnlineTracking)(const char* action,
                                                 const char* arg1,
                                                 const char* arg2,
                                                 const char* arg3,
                                                 const char* arg4,
                                                 const char* arg5);

typedef bool (QS_CALL_TYPE *FN_qsInit)(const qsSettings* settings);

#if defined(WIN32)
# ifdef _WIN64
QS_SELECTANY const char16_t* QS_MainDllPath = L"miniblink_x64.dll";
# else
QS_SELECTANY const char16_t* QS_MainDllPath = L"node.dll";
# endif

QS_SELECTANY HMODULE g_hMiniblinkMod = nullptr;

inline void qsSetMbMainDllPath(const char16_t* dllPath)
{
    QS_MainDllPath = dllPath;
}

inline void qsFillFuncPtr()
{
    if (!g_hMiniblinkMod) {
        g_hMiniblinkMod = LoadLibraryW(QS_MainDllPath);
    }

    if (!qsCreateWebView) {
        QS_FOR_EACH_DEFINE_FUNCTION(QS_GET_PTR_ITERATOR0, QS_GET_PTR_ITERATOR1, QS_GET_PTR_ITERATOR2, QS_GET_PTR_ITERATOR3, \
            QS_GET_PTR_ITERATOR4, QS_GET_PTR_ITERATOR5, QS_GET_PTR_ITERATOR6, QS_GET_PTR_ITERATOR7, QS_GET_PTR_ITERATOR8, QS_GET_PTR_ITERATOR9, QS_GET_PTR_ITERATOR10, QS_GET_PTR_ITERATOR11);
    }
}

inline void qsInit(const qsSettings* settings)
{
    bool needFill = nullptr == g_hMiniblinkMod;
    if (!g_hMiniblinkMod) {
        g_hMiniblinkMod = LoadLibraryW(QS_MainDllPath);
    }
    FN_qsInit qsInitExFunc = (FN_qsInit)GetProcAddress(g_hMiniblinkMod, "qsInit");
    qsInitExFunc(settings);

    if (needFill)
        qsFillFuncPtr();

    return;
}

#else // defined(WIN32)

static void* g_hMiniblinkMod = nullptr;

inline bool qsInit(const qsSettings* settings) {
    //printf("qsInit\n");
    if (!g_hMiniblinkMod) {
        g_hMiniblinkMod = dlopen("libqsframework.so", RTLD_LAZY);
        //printf("g_hMiniblinkMod: %p, %s \n", g_hMiniblinkMod, dlerror());
    }
    if (g_hMiniblinkMod) {
        FN_qsInit qsInitExFunc = (FN_qsInit)dlsym(g_hMiniblinkMod, "qsInit");
        //printf("qsInitExFunc: %p\n", qsInitExFunc);
        qsInitExFunc(settings);

        QS_FOR_EACH_DEFINE_FUNCTION(QS_GET_PTR_ITERATOR0, QS_GET_PTR_ITERATOR1, QS_GET_PTR_ITERATOR2, QS_GET_PTR_ITERATOR3, \
            QS_GET_PTR_ITERATOR4, QS_GET_PTR_ITERATOR5, QS_GET_PTR_ITERATOR6, QS_GET_PTR_ITERATOR7, QS_GET_PTR_ITERATOR8, \
            QS_GET_PTR_ITERATOR9, QS_GET_PTR_ITERATOR10, QS_GET_PTR_ITERATOR11);

        return true;
    }
    return false;
}

inline int QS_CALL_TYPE qsOnlineArgument(const char* key, char value[]) {
    if (!g_hMiniblinkMod) {
        return -1;
    }
    FN_qsOnlineArgument qsOnlineArgumentFunc = (FN_qsOnlineArgument)dlsym(g_hMiniblinkMod, "qsOnlineArgument");
    if (qsOnlineArgumentFunc) {
        return qsOnlineArgumentFunc(key, value);
    }
    return -1;
}

inline bool QS_CALL_TYPE qsOnlineTracking(const char* action,
                                          const char* arg1,
                                          const char* arg2,
                                          const char* arg3,
                                          const char* arg4,
                                          const char* arg5) {
    if (!g_hMiniblinkMod) {
        return false;
    }
    FN_qsOnlineTracking FN_qsOnlineTrackingFunc = (FN_qsOnlineTracking)dlsym(g_hMiniblinkMod, "qsOnlineTracking");
    if (FN_qsOnlineTrackingFunc) {
        return FN_qsOnlineTrackingFunc(action,
                                       arg1 ? arg1 : "",
                                       arg2 ? arg2 : "",
                                       arg3 ? arg3 : "",
                                       arg4 ? arg4 : "",
                                       arg5 ? arg5 : "");
    }
    return false;
}

#endif // defined(WIN32)

#endif // ENABLE_QS

#endif // QS_DEFINE_H

